@@ -248,7 +248,7 @@ where
 
 impl<T: ShMemProvider> Drop for RcShMem<T> {
     fn drop(&mut self) {
-        self.provider.borrow_mut().release_map(&mut self.internal)
+        self.provider.borrow_mut().release_map(&mut self.internal);
     }
 }
 
@@ -303,7 +303,7 @@ where
     }
 
     fn release_map(&mut self, map: &mut Self::Mem) {
-        self.internal.borrow_mut().release_map(&mut map.internal)
+        self.internal.borrow_mut().release_map(&mut map.internal);
     }
 
     fn clone_ref(&mut self, mapping: &Self::Mem) -> Result<Self::Mem, Error> {
@@ -656,7 +656,13 @@ pub mod unix_shmem {
                         if let Ok(boot_id) =
                             std::fs::read_to_string("/proc/sys/kernel/random/boot_id")
                         {
-                            format!("{}{}", "/dev/ashmem", boot_id).trim().to_string()
+                            let path_str =
+                                format!("{}{}", "/dev/ashmem", boot_id).trim().to_string();
+                            if std::path::Path::new(&path_str).exists() {
+                                path_str
+                            } else {
+                                "/dev/ashmem".to_string()
+                            }
                         } else {
                             "/dev/ashmem".to_string()
                         },
