@@ -15,15 +15,18 @@ use crate::bolts::os::unix_signals::setup_signal_handler;
 #[cfg(all(windows, feature = "std"))]
 use crate::bolts::os::windows_exceptions::setup_exception_handler;
 
+#[cfg(unix)]
+pub use unix_signal_handler::{nop_handler, HandlerFuncPtr};
+#[cfg(all(windows, feature = "std"))]
+pub use windows_exception_handler::{nop_handler, HandlerFuncPtr};
+
 use crate::{
     corpus::Corpus,
     events::{EventFirer, EventRestarter},
-    executors::{
-        Executor, ExitKind, HasExecHooks, HasExecHooksTuple, HasObservers, HasObserversHooks,
-    },
+    executors::{Executor, ExitKind, HasObservers},
     feedbacks::Feedback,
     fuzzer::HasObjective,
-    inputs::{HasTargetBytes, Input},
+    inputs::Input,
     observers::ObserversTuple,
     state::HasSolutions,
     Error,
@@ -32,61 +35,55 @@ use crate::{
 /// The inmem executor simply calls a target function, then returns afterwards.
 pub struct InProcessExecutor<'a, H, I, OT, S>
 where
-    H: FnMut(&[u8]) -> ExitKind,
-    I: Input + HasTargetBytes,
-    OT: ObserversTuple,
+    H: FnMut(&I) -> ExitKind,
+    I: Input,
+    OT: ObserversTuple<I, S>,
 {
     /// The harness function, being executed for each fuzzing loop execution
     harness_fn: &'a mut H,
     /// The observers, observing each run
     observers: OT,
+    /// On crash C function pointer
+    #[cfg(any(unix, all(windows, feature = "std")))]
+    crash_handler: HandlerFuncPtr,
+    /// On timeout C function pointer
+    #[cfg(unix)]
+    timeout_handler: HandlerFuncPtr,
     phantom: PhantomData<(I, S)>,
 }
 
-impl<'a, H, I, OT, S> Executor<I> for InProcessExecutor<'a, H, I, OT, S>
+impl<'a, EM, H, I, OT, S, Z> Executor<EM, I, S, Z> for InProcessExecutor<'a, H, I, OT, S>
 where
-    H: FnMut(&[u8]) -> ExitKind,
-    I: Input + HasTargetBytes,
-    OT: ObserversTuple,
+    H: FnMut(&I) -> ExitKind,
+    I: Input,
+    OT: ObserversTuple<I, S>,
 {
     #[inline]
-    fn run_target(&mut self, input: &I) -> Result<ExitKind, Error> {
-        let bytes = input.target_bytes();
-        let ret = (self.harness_fn)(bytes.as_slice());
-        Ok(ret)
-    }
-}
-
-impl<'a, EM, H, I, OT, S, Z> HasExecHooks<EM, I, S, Z> for InProcessExecutor<'a, H, I, OT, S>
-where
-    H: FnMut(&[u8]) -> ExitKind,
-    I: Input + HasTargetBytes,
-    OT: ObserversTuple,
-{
-    #[inline]
-    fn pre_exec(
+    fn run_target(
         &mut self,
-        _fuzzer: &mut Z,
-        _state: &mut S,
-        _event_mgr: &mut EM,
-        _input: &I,
-    ) -> Result<(), Error> {
+        fuzzer: &mut Z,
+        state: &mut S,
+        mgr: &mut EM,
+        input: &I,
+    ) -> Result<ExitKind, Error> {
         #[cfg(unix)]
         unsafe {
             let data = &mut unix_signal_handler::GLOBAL_STATE;
             write_volatile(
                 &mut data.current_input_ptr,
-                _input as *const _ as *const c_void,
+                input as *const _ as *const c_void,
             );
             write_volatile(
                 &mut data.observers_ptr,
                 &self.observers as *const _ as *const c_void,
             );
+            data.crash_handler = self.crash_handler;
+            data.timeout_handler = self.timeout_handler;
             // Direct raw pointers access /aliasing is pretty undefined behavior.
             // Since the state and event may have moved in memory, refresh them right before the signal may happen
-            write_volatile(&mut data.state_ptr, _state as *mut _ as *mut c_void);
-            write_volatile(&mut data.event_mgr_ptr, _event_mgr as *mut _ as *mut c_void);
-            write_volatile(&mut data.fuzzer_ptr, _fuzzer as *mut _ as *mut c_void);
+            write_volatile(&mut data.state_ptr, state as *mut _ as *mut c_void);
+            write_volatile(&mut data.event_mgr_ptr, mgr as *mut _ as *mut c_void);
+            write_volatile(&mut data.fuzzer_ptr, fuzzer as *mut _ as *mut c_void);
             compiler_fence(Ordering::SeqCst);
         }
         #[cfg(all(windows, feature = "std"))]
@@ -94,30 +91,24 @@ where
             let data = &mut windows_exception_handler::GLOBAL_STATE;
             write_volatile(
                 &mut data.current_input_ptr,
-                _input as *const _ as *const c_void,
+                input as *const _ as *const c_void,
             );
             write_volatile(
                 &mut data.observers_ptr,
                 &self.observers as *const _ as *const c_void,
             );
+            data.crash_handler = self.crash_handler;
+            //data.timeout_handler = self.timeout_handler;
             // Direct raw pointers access /aliasing is pretty undefined behavior.
             // Since the state and event may have moved in memory, refresh them right before the signal may happen
-            write_volatile(&mut data.state_ptr, _state as *mut _ as *mut c_void);
-            write_volatile(&mut data.event_mgr_ptr, _event_mgr as *mut _ as *mut c_void);
-            write_volatile(&mut data.fuzzer_ptr, _fuzzer as *mut _ as *mut c_void);
+            write_volatile(&mut data.state_ptr, state as *mut _ as *mut c_void);
+            write_volatile(&mut data.event_mgr_ptr, mgr as *mut _ as *mut c_void);
+            write_volatile(&mut data.fuzzer_ptr, fuzzer as *mut _ as *mut c_void);
             compiler_fence(Ordering::SeqCst);
         }
-        Ok(())
-    }
 
-    #[inline]
-    fn post_exec(
-        &mut self,
-        _fuzzer: &mut Z,
-        _state: &mut S,
-        _event_mgr: &mut EM,
-        _input: &I,
-    ) -> Result<(), Error> {
+        let ret = (self.harness_fn)(input);
+
         #[cfg(unix)]
         unsafe {
             write_volatile(
@@ -134,15 +125,16 @@ where
             );
             compiler_fence(Ordering::SeqCst);
         }
-        Ok(())
+
+        Ok(ret)
     }
 }
 
-impl<'a, H, I, OT, S> HasObservers<OT> for InProcessExecutor<'a, H, I, OT, S>
+impl<'a, H, I, OT, S> HasObservers<I, OT, S> for InProcessExecutor<'a, H, I, OT, S>
 where
-    H: FnMut(&[u8]) -> ExitKind,
-    I: Input + HasTargetBytes,
-    OT: ObserversTuple,
+    H: FnMut(&I) -> ExitKind,
+    I: Input,
+    OT: ObserversTuple<I, S>,
 {
     #[inline]
     fn observers(&self) -> &OT {
@@ -155,20 +147,11 @@ where
     }
 }
 
-impl<'a, EM, H, I, OT, S, Z> HasObserversHooks<EM, I, OT, S, Z>
-    for InProcessExecutor<'a, H, I, OT, S>
-where
-    H: FnMut(&[u8]) -> ExitKind,
-    I: Input + HasTargetBytes,
-    OT: ObserversTuple + HasExecHooksTuple<EM, I, S, Z>,
-{
-}
-
 impl<'a, H, I, OT, S> InProcessExecutor<'a, H, I, OT, S>
 where
-    H: FnMut(&[u8]) -> ExitKind,
-    I: Input + HasTargetBytes,
-    OT: ObserversTuple,
+    H: FnMut(&I) -> ExitKind,
+    I: Input,
+    OT: ObserversTuple<I, S>,
 {
     /// Create a new in mem executor.
     /// Caution: crash and restart in one of them will lead to odd behavior if multiple are used,
@@ -193,34 +176,48 @@ where
         #[cfg(unix)]
         unsafe {
             let data = &mut unix_signal_handler::GLOBAL_STATE;
-            write_volatile(
-                &mut data.crash_handler,
-                unix_signal_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S, Z>,
-            );
-            write_volatile(
-                &mut data.timeout_handler,
-                unix_signal_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S, Z>,
-            );
-
             setup_signal_handler(data)?;
             compiler_fence(Ordering::SeqCst);
+
+            Ok(Self {
+                harness_fn,
+                observers,
+                crash_handler: unix_signal_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S, Z>,
+                timeout_handler: unix_signal_handler::inproc_timeout_handler::<
+                    EM,
+                    I,
+                    OC,
+                    OF,
+                    OT,
+                    S,
+                    Z,
+                >,
+                phantom: PhantomData,
+            })
         }
         #[cfg(all(windows, feature = "std"))]
         unsafe {
             let data = &mut windows_exception_handler::GLOBAL_STATE;
-            write_volatile(
-                &mut data.crash_handler,
-                windows_exception_handler::inproc_crash_handler::<EM, I, OC, OF, OT, S, Z>,
-            );
-            //write_volatile(
-            //    &mut data.timeout_handler,
-            //    windows_exception_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S, Z>,
-            //);
-
             setup_exception_handler(data)?;
             compiler_fence(Ordering::SeqCst);
-        }
 
+            Ok(Self {
+                harness_fn,
+                observers,
+                crash_handler: windows_exception_handler::inproc_crash_handler::<
+                    EM,
+                    I,
+                    OC,
+                    OF,
+                    OT,
+                    S,
+                    Z,
+                >,
+                // timeout_handler: windows_exception_handler::inproc_timeout_handler::<EM, I, OC, OF, OT, S, Z>,
+                phantom: PhantomData,
+            })
+        }
+        #[cfg(not(any(unix, all(windows, feature = "std"))))]
         Ok(Self {
             harness_fn,
             observers,
@@ -231,7 +228,7 @@ where
     /// Retrieve the harness function.
     #[inline]
     pub fn harness(&self) -> &H {
-        &self.harness_fn
+        self.harness_fn
     }
 
     /// Retrieve the harness function for a mutable reference.
@@ -256,11 +253,14 @@ mod unix_signal_handler {
         executors::ExitKind,
         feedbacks::Feedback,
         fuzzer::HasObjective,
-        inputs::{HasTargetBytes, Input},
+        inputs::Input,
         observers::ObserversTuple,
         state::HasSolutions,
     };
 
+    pub type HandlerFuncPtr =
+        unsafe fn(Signal, siginfo_t, &mut ucontext_t, data: &mut InProcessExecutorHandlerData);
+
     // TODO merge GLOBAL_STATE with the Windows one
 
     /// Signal handling on unix systems needs some nasty unsafe.
@@ -287,14 +287,15 @@ mod unix_signal_handler {
         pub fuzzer_ptr: *mut c_void,
         pub observers_ptr: *const c_void,
         pub current_input_ptr: *const c_void,
-        pub crash_handler: unsafe fn(Signal, siginfo_t, &mut ucontext_t, data: &mut Self),
-        pub timeout_handler: unsafe fn(Signal, siginfo_t, &mut ucontext_t, data: &mut Self),
+        pub crash_handler: HandlerFuncPtr,
+        pub timeout_handler: HandlerFuncPtr,
     }
 
     unsafe impl Send for InProcessExecutorHandlerData {}
     unsafe impl Sync for InProcessExecutorHandlerData {}
 
-    unsafe fn nop_handler(
+    /// A handler that does nothing.
+    pub fn nop_handler(
         _signal: Signal,
         _info: siginfo_t,
         _context: &mut ucontext_t,
@@ -309,7 +310,7 @@ mod unix_signal_handler {
                 let data = &mut GLOBAL_STATE;
                 match signal {
                     Signal::SigUser2 | Signal::SigAlarm => {
-                        (data.timeout_handler)(signal, info, context, data)
+                        (data.timeout_handler)(signal, info, context, data);
                     }
                     _ => (data.crash_handler)(signal, info, context, data),
                 }
@@ -339,11 +340,11 @@ mod unix_signal_handler {
         data: &mut InProcessExecutorHandlerData,
     ) where
         EM: EventFirer<I, S> + EventRestarter<S>,
-        OT: ObserversTuple,
+        OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
         S: HasSolutions<OC, I>,
-        I: Input + HasTargetBytes,
+        I: Input,
         Z: HasObjective<I, OF, S>,
     {
         let state = (data.state_ptr as *mut S).as_mut().unwrap();
@@ -365,7 +366,7 @@ mod unix_signal_handler {
 
             let interesting = fuzzer
                 .objective_mut()
-                .is_interesting(state, &input, observers, &ExitKind::Timeout)
+                .is_interesting(state, event_mgr, input, observers, &ExitKind::Timeout)
                 .expect("In timeout handler objective failure.");
 
             if interesting {
@@ -413,11 +414,11 @@ mod unix_signal_handler {
         data: &mut InProcessExecutorHandlerData,
     ) where
         EM: EventFirer<I, S> + EventRestarter<S>,
-        OT: ObserversTuple,
+        OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
         S: HasSolutions<OC, I>,
-        I: Input + HasTargetBytes,
+        I: Input,
         Z: HasObjective<I, OF, S>,
     {
         #[cfg(all(target_os = "android", target_arch = "aarch64"))]
@@ -431,8 +432,7 @@ mod unix_signal_handler {
             {
                 println!("Double crash\n");
                 #[cfg(target_os = "android")]
-                let si_addr =
-                    { ((_info._pad[0] as usize) | ((_info._pad[1] as usize) << 32)) as usize };
+                let si_addr = (_info._pad[0] as i64) | ((_info._pad[1] as i64) << 32);
                 #[cfg(not(target_os = "android"))]
                 let si_addr = { _info.si_addr() as usize };
 
@@ -457,7 +457,6 @@ mod unix_signal_handler {
             }
 
             // TODO tell the parent to not restart
-            libc::_exit(1);
         } else {
             let state = (data.state_ptr as *mut S).as_mut().unwrap();
             let event_mgr = (data.event_mgr_ptr as *mut EM).as_mut().unwrap();
@@ -467,27 +466,18 @@ mod unix_signal_handler {
             #[cfg(feature = "std")]
             println!("Child crashed!");
 
+            #[allow(clippy::non_ascii_literal)]
             #[cfg(all(
                 feature = "std",
                 any(target_os = "linux", target_os = "android"),
                 target_arch = "aarch64"
             ))]
             {
-                use crate::bolts::os::find_mapping_for_address;
                 println!("{:━^100}", " CRASH ");
                 println!(
                     "Received signal {} at 0x{:016x}, fault address: 0x{:016x}",
                     _signal, _context.uc_mcontext.pc, _context.uc_mcontext.fault_address
                 );
-                if let Ok((start, _, _, path)) =
-                    find_mapping_for_address(_context.uc_mcontext.pc as usize)
-                {
-                    println!(
-                        "pc is at offset 0x{:08x} in  {}",
-                        _context.uc_mcontext.pc as usize - start,
-                        path
-                    );
-                }
 
                 println!("{:━^100}", " REGISTERS ");
                 for reg in 0..31 {
@@ -514,7 +504,7 @@ mod unix_signal_handler {
 
             let interesting = fuzzer
                 .objective_mut()
-                .is_interesting(state, &input, observers, &ExitKind::Crash)
+                .is_interesting(state, event_mgr, input, observers, &ExitKind::Crash)
                 .expect("In crash handler objective failure.");
 
             if interesting {
@@ -545,9 +535,9 @@ mod unix_signal_handler {
             event_mgr.await_restart_safe();
             #[cfg(feature = "std")]
             println!("Bye!");
-
-            libc::_exit(1);
         }
+
+        libc::_exit(1);
     }
 }
 
@@ -570,11 +560,14 @@ mod windows_exception_handler {
         executors::ExitKind,
         feedbacks::Feedback,
         fuzzer::HasObjective,
-        inputs::{HasTargetBytes, Input},
+        inputs::Input,
         observers::ObserversTuple,
         state::HasSolutions,
     };
 
+    pub type HandlerFuncPtr =
+        unsafe fn(ExceptionCode, *mut EXCEPTION_POINTERS, &mut InProcessExecutorHandlerData);
+
     /// Signal handling on unix systems needs some nasty unsafe.
     pub static mut GLOBAL_STATE: InProcessExecutorHandlerData = InProcessExecutorHandlerData {
         /// The state ptr for signal handling
@@ -599,14 +592,14 @@ mod windows_exception_handler {
         pub fuzzer_ptr: *mut c_void,
         pub observers_ptr: *const c_void,
         pub current_input_ptr: *const c_void,
-        pub crash_handler: unsafe fn(ExceptionCode, *mut EXCEPTION_POINTERS, &mut Self),
-        //pub timeout_handler: unsafe fn(ExceptionCode, *mut EXCEPTION_POINTERS, &mut Self),
+        pub crash_handler: HandlerFuncPtr,
+        //pub timeout_handler: HandlerFuncPtr,
     }
 
     unsafe impl Send for InProcessExecutorHandlerData {}
     unsafe impl Sync for InProcessExecutorHandlerData {}
 
-    unsafe fn nop_handler(
+    pub unsafe fn nop_handler(
         _code: ExceptionCode,
         _exception_pointers: *mut EXCEPTION_POINTERS,
         _data: &mut InProcessExecutorHandlerData,
@@ -632,11 +625,11 @@ mod windows_exception_handler {
         data: &mut InProcessExecutorHandlerData,
     ) where
         EM: EventFirer<I, S> + EventRestarter<S>,
-        OT: ObserversTuple,
+        OT: ObserversTuple<I, S>,
         OC: Corpus<I>,
         OF: Feedback<I, S>,
         S: HasSolutions<OC, I>,
-        I: Input + HasTargetBytes,
+        I: Input,
         Z: HasObjective<I, OF, S>,
     {
         #[cfg(feature = "std")]
@@ -658,7 +651,7 @@ mod windows_exception_handler {
 
             let interesting = fuzzer
                 .objective_mut()
-                .is_interesting(state, &input, observers, &ExitKind::Crash)
+                .is_interesting(state, event_mgr, &input, observers, &ExitKind::Crash)
                 .expect("In crash handler objective failure.");
 
             if interesting {
@@ -729,20 +722,26 @@ mod tests {
 
     use crate::{
         bolts::tuples::tuple_list,
-        executors::{Executor, ExitKind, InProcessExecutor},
+        executors::{inprocess, Executor, ExitKind, InProcessExecutor},
         inputs::NopInput,
     };
 
     #[test]
     fn test_inmem_exec() {
-        let mut harness = |_buf: &[u8]| ExitKind::Ok;
+        let mut harness = |_buf: &NopInput| ExitKind::Ok;
 
         let mut in_process_executor = InProcessExecutor::<_, NopInput, (), ()> {
             harness_fn: &mut harness,
             observers: tuple_list!(),
+            #[cfg(any(unix, all(windows, feature = "std")))]
+            crash_handler: inprocess::nop_handler,
+            #[cfg(unix)]
+            timeout_handler: inprocess::nop_handler,
             phantom: PhantomData,
         };
         let input = NopInput {};
-        assert!(in_process_executor.run_target(&input).is_ok());
+        assert!(in_process_executor
+            .run_target(&mut (), &mut (), &mut (), &input)
+            .is_ok());
     }
 }
