@@ -11,18 +11,20 @@ use serde::{Deserialize, Serialize};
 use crate::{
     bolts::{tuples::Named, AsSlice},
     corpus::Testcase,
+    events::{Event, EventFirer},
     executors::ExitKind,
     feedbacks::{Feedback, FeedbackState, FeedbackStatesTuple},
     inputs::Input,
     observers::{MapObserver, ObserversTuple},
     state::{HasFeedbackStates, HasMetadata},
+    stats::UserStats,
     Error,
 };
 
 /// A [`MapFeedback`] that strives to maximize the map contents.
-pub type MaxMapFeedback<FT, O, S, T> = MapFeedback<FT, O, MaxReducer, S, T>;
+pub type MaxMapFeedback<FT, I, O, S, T> = MapFeedback<FT, I, O, MaxReducer, S, T>;
 /// A [`MapFeedback`] that strives to minimize the map contents.
-pub type MinMapFeedback<FT, O, S, T> = MapFeedback<FT, O, MinReducer, S, T>;
+pub type MinMapFeedback<FT, I, O, S, T> = MapFeedback<FT, I, O, MinReducer, S, T>;
 
 /// A Reducer function is used to aggregate values for the novelty search
 pub trait Reducer<T>: Serialize + serde::de::DeserializeOwned + 'static
@@ -183,7 +185,7 @@ where
 /// The most common AFL-like feedback type
 #[derive(Serialize, Deserialize, Clone, Debug)]
 #[serde(bound = "T: serde::de::DeserializeOwned")]
-pub struct MapFeedback<FT, O, R, S, T>
+pub struct MapFeedback<FT, I, O, R, S, T>
 where
     T: Integer + Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
     R: Reducer<T>,
@@ -200,10 +202,10 @@ where
     /// Name identifier of the observer
     observer_name: String,
     /// Phantom Data of Reducer
-    phantom: PhantomData<(FT, S, R, O, T)>,
+    phantom: PhantomData<(FT, I, S, R, O, T)>,
 }
 
-impl<I, FT, O, R, S, T> Feedback<I, S> for MapFeedback<FT, O, R, S, T>
+impl<I, FT, O, R, S, T> Feedback<I, S> for MapFeedback<FT, I, O, R, S, T>
 where
     T: Integer + Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
     R: Reducer<T>,
@@ -212,15 +214,17 @@ where
     S: HasFeedbackStates<FT>,
     FT: FeedbackStatesTuple,
 {
-    fn is_interesting<OT>(
+    fn is_interesting<EM, OT>(
         &mut self,
         state: &mut S,
+        manager: &mut EM,
         _input: &I,
         observers: &OT,
         _exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        OT: ObserversTuple,
+        EM: EventFirer<I, S>,
+        OT: ObserversTuple<I, S>,
     {
         let mut interesting = false;
         // TODO Replace with match_name_type when stable
@@ -233,32 +237,10 @@ where
             .match_name_mut::<MapFeedbackState<T>>(&self.name)
             .unwrap();
 
-        if self.indexes.is_none() && self.novelties.is_none() {
-            for i in 0..size {
-                let history = map_state.history_map[i];
-                let item = observer.map()[i];
+        assert!(size <= map_state.history_map.len());
+        assert!(size <= observer.map().len());
 
-                let reduced = R::reduce(history, item);
-                if history != reduced {
-                    map_state.history_map[i] = reduced;
-                    interesting = true;
-                }
-            }
-        } else if self.indexes.is_some() && self.novelties.is_none() {
-            for i in 0..size {
-                let history = map_state.history_map[i];
-                let item = observer.map()[i];
-                if item != initial {
-                    self.indexes.as_mut().unwrap().push(i);
-                }
-
-                let reduced = R::reduce(history, item);
-                if history != reduced {
-                    map_state.history_map[i] = reduced;
-                    interesting = true;
-                }
-            }
-        } else if self.indexes.is_none() && self.novelties.is_some() {
+        if self.novelties.is_some() {
             for i in 0..size {
                 let history = map_state.history_map[i];
                 let item = observer.map()[i];
@@ -274,19 +256,35 @@ where
             for i in 0..size {
                 let history = map_state.history_map[i];
                 let item = observer.map()[i];
-                if item != initial {
-                    self.indexes.as_mut().unwrap().push(i);
-                }
 
                 let reduced = R::reduce(history, item);
                 if history != reduced {
                     map_state.history_map[i] = reduced;
                     interesting = true;
-                    self.novelties.as_mut().unwrap().push(i);
                 }
             }
         }
 
+        if interesting {
+            let mut filled = 0;
+            for i in 0..size {
+                if map_state.history_map[i] != initial {
+                    filled += 1;
+                    if self.indexes.is_some() {
+                        self.indexes.as_mut().unwrap().push(i);
+                    }
+                }
+            }
+            manager.fire(
+                state,
+                Event::UpdateUserStats {
+                    name: self.name.to_string(),
+                    value: UserStats::Ratio(filled, size as u64),
+                    phantom: PhantomData,
+                },
+            )?;
+        }
+
         Ok(interesting)
     }
 
@@ -314,7 +312,7 @@ where
     }
 }
 
-impl<FT, O, R, S, T> Named for MapFeedback<FT, O, R, S, T>
+impl<FT, I, O, R, S, T> Named for MapFeedback<FT, I, O, R, S, T>
 where
     T: Integer + Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
     R: Reducer<T>,
@@ -328,7 +326,7 @@ where
     }
 }
 
-impl<FT, O, R, S, T> MapFeedback<FT, O, R, S, T>
+impl<FT, I, O, R, S, T> MapFeedback<FT, I, O, R, S, T>
 where
     T: Integer + Default + Copy + 'static + serde::Serialize + serde::de::DeserializeOwned,
     R: Reducer<T>,
@@ -433,15 +431,17 @@ where
     I: Input,
     O: MapObserver<usize>,
 {
-    fn is_interesting<OT>(
+    fn is_interesting<EM, OT>(
         &mut self,
         _state: &mut S,
+        _manager: &mut EM,
         _input: &I,
         observers: &OT,
         _exit_kind: &ExitKind,
     ) -> Result<bool, Error>
     where
-        OT: ObserversTuple,
+        EM: EventFirer<I, S>,
+        OT: ObserversTuple<I, S>,
     {
         // TODO Replace with match_name_type when stable
         let observer = observers.match_name::<O>(&self.name).unwrap();
