@@ -3,10 +3,8 @@
 use crate::{
     bolts::current_time,
     corpus::{Corpus, CorpusScheduler, Testcase},
-    events::{Event, EventManager},
-    executors::{
-        Executor, ExitKind, HasExecHooks, HasExecHooksTuple, HasObservers, HasObserversHooks,
-    },
+    events::{Event, EventFirer, EventManager},
+    executors::{Executor, ExitKind, HasObservers},
     feedbacks::Feedback,
     inputs::Input,
     mark_feature_time,
@@ -19,6 +17,8 @@ use crate::{
 
 #[cfg(feature = "introspection")]
 use crate::stats::PerfFeature;
+#[cfg(feature = "introspection")]
+use alloc::boxed::Box;
 
 use alloc::string::ToString;
 use core::{marker::PhantomData, time::Duration};
@@ -66,29 +66,44 @@ where
 }
 
 /// Evaluate if an input is interesting using the feedback
-pub trait IsInteresting<I, OT, S>
+pub trait ExecutionProcessor<I, OT, S>
 where
-    OT: ObserversTuple,
+    OT: ObserversTuple<I, S>,
+    I: Input,
 {
     /// Evaluate if a set of observation channels has an interesting state
-    fn is_interesting(
+    fn process_execution<EM>(
         &mut self,
         state: &mut S,
-        input: &I,
+        manager: &mut EM,
+        input: I,
         observers: &OT,
         exit_kind: &ExitKind,
-    ) -> Result<bool, Error>;
+        send_events: bool,
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error>
+    where
+        EM: EventFirer<I, S>;
 }
 
-/// Add to the state if interesting
-pub trait IfInteresting<I, S> {
-    /// Adds this input to the corpus, if it's intersting, and return the index
-    fn add_if_interesting(
+/// Evaluate an input modyfing the state of the fuzzer
+pub trait EvaluatorObservers<I, OT, S>: Sized
+where
+    I: Input,
+    OT: ObserversTuple<I, S>,
+{
+    /// Runs the input and triggers observers and feedback,
+    /// returns if is interesting an (option) the index of the new testcase in the corpus
+    fn evaluate_input_with_observers<E, EM>(
         &mut self,
         state: &mut S,
-        input: &I,
-        is_interesting: bool,
-    ) -> Result<Option<usize>, Error>;
+        executor: &mut E,
+        manager: &mut EM,
+        input: I,
+        send_events: bool,
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error>
+    where
+        E: Executor<EM, I, S, Self> + HasObservers<I, OT, S>,
+        EM: EventManager<E, I, S, Self>;
 }
 
 /// Evaluate an input modyfing the state of the fuzzer
@@ -101,7 +116,33 @@ pub trait Evaluator<E, EM, I, S> {
         executor: &mut E,
         manager: &mut EM,
         input: I,
-    ) -> Result<(bool, Option<usize>), Error>;
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error> {
+        self.evaluate_input_events(state, executor, manager, input, true)
+    }
+
+    /// Runs the input and triggers observers and feedback,
+    /// returns if is interesting an (option) the index of the new testcase in the corpus
+    /// This version has a boolean to decide if send events to the manager.
+    fn evaluate_input_events(
+        &mut self,
+        state: &mut S,
+        executor: &mut E,
+        manager: &mut EM,
+        input: I,
+        send_events: bool,
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error>;
+
+    /// Runs the input and triggers observers and feedback.
+    /// Adds an input, to the corpus even if it's not considered `interesting` by the `feedback`.
+    /// Returns the `index` of the new testcase in the corpus.
+    /// Usually, you want to use [`Evaluator::evaluate_input`], unless you know what you are doing.
+    fn add_input(
+        &mut self,
+        state: &mut S,
+        executor: &mut E,
+        manager: &mut EM,
+        input: I,
+    ) -> Result<usize, Error>;
 }
 
 /// The main fuzzer trait.
@@ -184,6 +225,13 @@ pub trait Fuzzer<E, EM, I, S, ST> {
     ) -> Result<Duration, Error>;
 }
 
+#[derive(Debug, PartialEq)]
+pub enum ExecuteInputResult {
+    None,
+    Corpus,
+    Solution,
+}
+
 /// Your default fuzzer instance, for everyday use.
 #[derive(Debug)]
 pub struct StdFuzzer<C, CS, F, I, OF, OT, S, SC>
@@ -248,60 +296,165 @@ where
     }
 }
 
-impl<C, CS, F, I, OF, OT, S, SC> IsInteresting<I, OT, S> for StdFuzzer<C, CS, F, I, OF, OT, S, SC>
+impl<C, CS, F, I, OF, OT, S, SC> ExecutionProcessor<I, OT, S>
+    for StdFuzzer<C, CS, F, I, OF, OT, S, SC>
 where
     C: Corpus<I>,
+    SC: Corpus<I>,
     CS: CorpusScheduler<I, S>,
     F: Feedback<I, S>,
     I: Input,
     OF: Feedback<I, S>,
-    OT: ObserversTuple,
-    S: HasCorpus<C, I>,
+    OT: ObserversTuple<I, S>,
+    S: HasCorpus<C, I> + HasSolutions<SC, I> + HasClientPerfStats + HasExecutions,
 {
     /// Evaluate if a set of observation channels has an interesting state
-    fn is_interesting(
+    fn process_execution<EM>(
         &mut self,
         state: &mut S,
-        input: &I,
+        manager: &mut EM,
+        input: I,
         observers: &OT,
         exit_kind: &ExitKind,
-    ) -> Result<bool, Error>
+        send_events: bool,
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error>
     where
-        OT: ObserversTuple,
+        EM: EventFirer<I, S>,
     {
-        self.feedback_mut()
-            .is_interesting(state, input, observers, exit_kind)
+        let mut res = ExecuteInputResult::None;
+
+        start_timer!(state);
+        let is_solution = self
+            .objective_mut()
+            .is_interesting(state, manager, &input, observers, &exit_kind)?;
+        mark_feature_time!(state, PerfFeature::GetObjectivesInterestingAll);
+
+        if is_solution {
+            res = ExecuteInputResult::Solution;
+        } else {
+            #[cfg(not(feature = "introspection"))]
+            let is_corpus = self
+                .feedback_mut()
+                .is_interesting(state, manager, &input, observers, &exit_kind)?;
+
+            #[cfg(feature = "introspection")]
+            let is_corpus = {
+                // Init temporary feedback stats here. We can't use the typical pattern above
+                // since we need a `mut self` for `feedbacks_mut`, so we can't also hand a
+                // new `mut self` to `is_interesting_with_perf`. We use this stack
+                // variable to get the stats and then update the feedbacks directly
+                let mut feedback_stats = [0_u64; crate::stats::NUM_FEEDBACKS];
+                let feedback_index = 0;
+                let is_corpus = self.feedback_mut().is_interesting_with_perf(
+                    state,
+                    manager,
+                    &input,
+                    observers,
+                    &exit_kind,
+                    &mut feedback_stats,
+                    feedback_index,
+                )?;
+
+                // Update the feedback stats
+                state
+                    .introspection_stats_mut()
+                    .update_feedbacks(feedback_stats);
+
+                // Return the total fitness
+                is_corpus
+            };
+
+            if is_corpus {
+                res = ExecuteInputResult::Corpus;
+            }
+        }
+
+        match res {
+            ExecuteInputResult::None => {
+                self.feedback_mut().discard_metadata(state, &input)?;
+                self.objective_mut().discard_metadata(state, &input)?;
+                Ok((res, None))
+            }
+            ExecuteInputResult::Corpus => {
+                // Not a solution
+                self.objective_mut().discard_metadata(state, &input)?;
+
+                // Add the input to the main corpus
+                let mut testcase = Testcase::new(input.clone());
+                self.feedback_mut().append_metadata(state, &mut testcase)?;
+                let idx = state.corpus_mut().add(testcase)?;
+                self.scheduler_mut().on_add(state, idx)?;
+
+                if send_events {
+                    let observers_buf = manager.serialize_observers(observers)?;
+                    manager.fire(
+                        state,
+                        Event::NewTestcase {
+                            input,
+                            observers_buf,
+                            exit_kind: exit_kind.clone(),
+                            corpus_size: state.corpus().count(),
+                            client_config: manager.configuration().to_string(),
+                            time: current_time(),
+                            executions: *state.executions(),
+                        },
+                    )?;
+                }
+                Ok((res, Some(idx)))
+            }
+            ExecuteInputResult::Solution => {
+                // Not interesting
+                self.feedback_mut().discard_metadata(state, &input)?;
+
+                // The input is a solution, add it to the respective corpus
+                let mut testcase = Testcase::new(input);
+                self.objective_mut().append_metadata(state, &mut testcase)?;
+                state.solutions_mut().add(testcase)?;
+
+                if send_events {
+                    manager.fire(
+                        state,
+                        Event::Objective {
+                            objective_size: state.solutions().count(),
+                        },
+                    )?;
+                }
+
+                Ok((res, None))
+            }
+        }
     }
 }
 
-impl<C, CS, F, I, OF, OT, S, SC> IfInteresting<I, S> for StdFuzzer<C, CS, F, I, OF, OT, S, SC>
+impl<C, CS, F, I, OF, OT, S, SC> EvaluatorObservers<I, OT, S>
+    for StdFuzzer<C, CS, F, I, OF, OT, S, SC>
 where
     C: Corpus<I>,
     CS: CorpusScheduler<I, S>,
+    OT: ObserversTuple<I, S>,
     F: Feedback<I, S>,
     I: Input,
     OF: Feedback<I, S>,
-    OT: ObserversTuple,
-    S: HasCorpus<C, I>,
+    S: HasExecutions + HasCorpus<C, I> + HasSolutions<SC, I> + HasClientPerfStats,
+    SC: Corpus<I>,
 {
-    /// Adds this input to the corpus, if it's intersting, and return the index
+    /// Process one input, adding to the respective corpuses if needed and firing the right events
     #[inline]
-    fn add_if_interesting(
+    fn evaluate_input_with_observers<E, EM>(
         &mut self,
         state: &mut S,
-        input: &I,
-        is_interesting: bool,
-    ) -> Result<Option<usize>, Error> {
-        if is_interesting {
-            let mut testcase = Testcase::new(input.clone());
-            self.feedback_mut().append_metadata(state, &mut testcase)?;
-            let idx = state.corpus_mut().add(testcase)?;
-            self.scheduler_mut().on_add(state, idx)?;
-            Ok(Some(idx))
-        } else {
-            self.feedback_mut().discard_metadata(state, input)?;
-            Ok(None)
-        }
+        executor: &mut E,
+        manager: &mut EM,
+        input: I,
+        send_events: bool,
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error>
+    where
+        E: Executor<EM, I, S, Self> + HasObservers<I, OT, S>,
+        EM: EventManager<E, I, S, Self>,
+    {
+        let exit_kind = self.execute_input(state, executor, manager, &input)?;
+        let observers = executor.observers();
+        self.process_execution(state, manager, input, observers, &exit_kind, send_events)
     }
 }
 
@@ -310,11 +463,8 @@ impl<C, CS, E, EM, F, I, OF, OT, S, SC> Evaluator<E, EM, I, S>
 where
     C: Corpus<I>,
     CS: CorpusScheduler<I, S>,
-    E: Executor<I>
-        + HasObservers<OT>
-        + HasExecHooks<EM, I, S, Self>
-        + HasObserversHooks<EM, I, OT, S, Self>,
-    OT: ObserversTuple + HasExecHooksTuple<EM, I, S, Self>,
+    E: Executor<EM, I, S, Self> + HasObservers<I, OT, S>,
+    OT: ObserversTuple<I, S>,
     EM: EventManager<E, I, S, Self>,
     F: Feedback<I, S>,
     I: Input,
@@ -324,42 +474,52 @@ where
 {
     /// Process one input, adding to the respective corpuses if needed and firing the right events
     #[inline]
-    fn evaluate_input(
+    fn evaluate_input_events(
         &mut self,
         state: &mut S,
         executor: &mut E,
         manager: &mut EM,
         input: I,
-    ) -> Result<(bool, Option<usize>), Error> {
-        let (is_interesting, is_solution) = self.execute_input(state, executor, manager, &input)?;
+        send_events: bool,
+    ) -> Result<(ExecuteInputResult, Option<usize>), Error> {
+        self.evaluate_input_with_observers(state, executor, manager, input, send_events)
+    }
+
+    /// Adds an input, even if it's not conisered `interesting` by any of the executors
+    fn add_input(
+        &mut self,
+        state: &mut S,
+        executor: &mut E,
+        manager: &mut EM,
+        input: I,
+    ) -> Result<usize, Error> {
+        let exit_kind = self.execute_input(state, executor, manager, &input)?;
         let observers = executor.observers();
+        // Always consider this to be "interesting"
 
-        if is_solution {
-            // If the input is a solution, add it to the respective corpus
-            let mut testcase = Testcase::new(input.clone());
-            self.objective_mut().append_metadata(state, &mut testcase)?;
-            state.solutions_mut().add(testcase)?;
-        } else {
-            self.objective_mut().discard_metadata(state, &input)?;
-        }
+        // Not a solution
+        self.objective_mut().discard_metadata(state, &input)?;
 
-        let corpus_idx = self.add_if_interesting(state, &input, is_interesting)?;
-        if corpus_idx.is_some() {
-            let observers_buf = manager.serialize_observers(observers)?;
-            manager.fire(
-                state,
-                Event::NewTestcase {
-                    input,
-                    observers_buf,
-                    corpus_size: state.corpus().count(),
-                    client_config: "TODO".into(),
-                    time: current_time(),
-                    executions: *state.executions(),
-                },
-            )?;
-        }
+        // Add the input to the main corpus
+        let mut testcase = Testcase::new(input.clone());
+        self.feedback_mut().append_metadata(state, &mut testcase)?;
+        let idx = state.corpus_mut().add(testcase)?;
+        self.scheduler_mut().on_add(state, idx)?;
 
-        Ok((is_interesting, corpus_idx))
+        let observers_buf = manager.serialize_observers(observers)?;
+        manager.fire(
+            state,
+            Event::NewTestcase {
+                input,
+                observers_buf,
+                exit_kind,
+                corpus_size: state.corpus().count(),
+                client_config: manager.configuration().to_string(),
+                time: current_time(),
+                executions: *state.executions(),
+            },
+        )?;
+        Ok(idx)
     }
 }
 
@@ -382,7 +542,8 @@ where
         stats_timeout: Duration,
     ) -> Result<Duration, Error> {
         let cur = current_time();
-        if cur - last > stats_timeout {
+        // default to 0 here to avoid crashes on clock skew
+        if cur.checked_sub(last).unwrap_or_default() > stats_timeout {
             // Default no introspection implmentation
             #[cfg(not(feature = "introspection"))]
             manager.fire(
@@ -399,7 +560,7 @@ where
             {
                 state
                     .introspection_stats_mut()
-                    .set_current_time(crate::cpu::read_time_counter());
+                    .set_current_time(crate::bolts::cpu::read_time_counter());
 
                 // Send the current stats over to the manager. This `.clone` shouldn't be
                 // costly as `ClientPerfStats` impls `Copy` since it only contains `u64`s
@@ -486,76 +647,25 @@ where
         executor: &mut E,
         event_mgr: &mut EM,
         input: &I,
-    ) -> Result<(bool, bool), Error>
+    ) -> Result<ExitKind, Error>
     where
-        E: Executor<I>
-            + HasObservers<OT>
-            + HasExecHooks<EM, I, S, Self>
-            + HasObserversHooks<EM, I, OT, S, Self>,
-        OT: ObserversTuple + HasExecHooksTuple<EM, I, S, Self>,
-        EM: EventManager<E, I, S, Self>,
+        E: Executor<EM, I, S, Self> + HasObservers<I, OT, S>,
+        OT: ObserversTuple<I, S>,
     {
         start_timer!(state);
-        executor.pre_exec_observers(self, state, event_mgr, input)?;
+        executor.observers_mut().pre_exec_all(state, input)?;
         mark_feature_time!(state, PerfFeature::PreExecObservers);
 
         start_timer!(state);
-        executor.pre_exec(self, state, event_mgr, input)?;
-        mark_feature_time!(state, PerfFeature::PreExec);
-
-        start_timer!(state);
-        let exit_kind = executor.run_target(input)?;
+        let exit_kind = executor.run_target(self, state, event_mgr, input)?;
         mark_feature_time!(state, PerfFeature::TargetExecution);
 
-        start_timer!(state);
-        executor.post_exec(self, state, event_mgr, input)?;
-        mark_feature_time!(state, PerfFeature::PostExec);
-
         *state.executions_mut() += 1;
 
         start_timer!(state);
-        executor.post_exec_observers(self, state, event_mgr, input)?;
+        executor.observers_mut().post_exec_all(state, input)?;
         mark_feature_time!(state, PerfFeature::PostExecObservers);
 
-        let observers = executor.observers();
-        #[cfg(not(feature = "introspection"))]
-        let is_interesting = self
-            .feedback_mut()
-            .is_interesting(state, &input, observers, &exit_kind)?;
-
-        #[cfg(feature = "introspection")]
-        let is_interesting = {
-            // Init temporary feedback stats here. We can't use the typical pattern above
-            // since we need a `mut self` for `feedbacks_mut`, so we can't also hand a
-            // new `mut self` to `is_interesting_with_perf`. We use this stack
-            // variable to get the stats and then update the feedbacks directly
-            let mut feedback_stats = [0_u64; crate::stats::NUM_FEEDBACKS];
-            let feedback_index = 0;
-            let is_interesting = self.feedback_mut().is_interesting_with_perf(
-                state,
-                &input,
-                observers,
-                &exit_kind,
-                &mut feedback_stats,
-                feedback_index,
-            )?;
-
-            // Update the feedback stats
-            state
-                .introspection_stats_mut()
-                .update_feedbacks(feedback_stats);
-
-            // Return the total fitness
-            is_interesting
-        };
-
-        start_timer!(state);
-        let is_solution = self
-            .objective_mut()
-            .is_interesting(state, &input, observers, &exit_kind)?;
-
-        mark_feature_time!(state, PerfFeature::GetObjectivesInterestingAll);
-
-        Ok((is_interesting, is_solution))
+        Ok(exit_kind)
     }
 }
