@@ -25,7 +25,6 @@ pub use libafl_derive::*;
 
 pub mod bolts;
 pub mod corpus;
-pub mod cpu;
 pub mod events;
 pub mod executors;
 pub mod feedbacks;
@@ -53,7 +52,7 @@ pub enum Error {
     Serialize(String),
     /// Compression error
     #[cfg(feature = "llmp_compression")]
-    Compression(String),
+    Compression,
     /// File related error
     #[cfg(feature = "std")]
     File(io::Error),
@@ -71,6 +70,8 @@ pub enum Error {
     IllegalState(String),
     /// The argument passed to this method or function is not valid
     IllegalArgument(String),
+    /// Forkserver related Error
+    Forkserver(String),
     /// Shutting down, not really an error.
     ShuttingDown,
     /// Something else happened
@@ -82,7 +83,7 @@ impl fmt::Display for Error {
         match self {
             Self::Serialize(s) => write!(f, "Error in Serialization: `{0}`", &s),
             #[cfg(feature = "llmp_compression")]
-            Self::Compression(s) => write!(f, "Error in Compression: `{0}`", &s),
+            Self::Compression => write!(f, "Error in decompression"),
             #[cfg(feature = "std")]
             Self::File(err) => write!(f, "File IO failed: {:?}", &err),
             Self::EmptyOptional(s) => write!(f, "Optional value `{0}` was not set", &s),
@@ -94,6 +95,7 @@ impl fmt::Display for Error {
             Self::NotImplemented(s) => write!(f, "Not implemented: {0}", &s),
             Self::IllegalState(s) => write!(f, "Illegal state: {0}", &s),
             Self::IllegalArgument(s) => write!(f, "Illegal argument: {0}", &s),
+            Self::Forkserver(s) => write!(f, "Forkserver : {0}", &s),
             Self::ShuttingDown => write!(f, "Shutting down!"),
             Self::Unknown(s) => write!(f, "Unknown error: {0}", &s),
         }
@@ -107,13 +109,6 @@ impl From<postcard::Error> for Error {
     }
 }
 
-#[cfg(feature = "llmp_compression")]
-impl From<compression::prelude::CompressionError> for Error {
-    fn from(err: compression::prelude::CompressionError) -> Self {
-        Self::Compression(format!("{:?}", err))
-    }
-}
-
 /// Stringify the json serializer error
 #[cfg(feature = "std")]
 impl From<serde_json::Error> for Error {
@@ -201,7 +196,7 @@ mod tests {
         let scheduler = RandCorpusScheduler::new();
         let mut fuzzer = StdFuzzer::new(scheduler, (), ());
 
-        let mut harness = |_buf: &[u8]| ExitKind::Ok;
+        let mut harness = |_buf: &BytesInput| ExitKind::Ok;
         let mut executor = InProcessExecutor::new(
             &mut harness,
             tuple_list!(),
