@@ -1,9 +1,14 @@
 //! Keep stats, and dispaly them to the user. Usually used in a broker, or main node, of some sort.
 
+pub mod multi;
+pub use multi::MultiStats;
+
 use serde::{Deserialize, Serialize};
 
 use alloc::{string::String, vec::Vec};
-use core::{time, time::Duration};
+use core::{fmt, time, time::Duration};
+
+use hashbrown::HashMap;
 
 #[cfg(feature = "introspection")]
 use alloc::string::ToString;
@@ -14,6 +19,36 @@ use crate::bolts::current_time;
 
 const CLIENT_STATS_TIME_WINDOW_SECS: u64 = 5; // 5 seconds
 
+/// Number of stages in the fuzzer
+pub(crate) const NUM_STAGES: usize = 8;
+
+/// Number of feedback mechanisms to measure for performance
+pub(crate) const NUM_FEEDBACKS: usize = 16;
+
+/// User-defined stats types
+#[derive(Serialize, Deserialize, Debug, Clone)]
+pub enum UserStats {
+    Number(u64),
+    String(String),
+    Ratio(u64, u64),
+}
+
+impl fmt::Display for UserStats {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            UserStats::Number(n) => write!(f, "{}", n),
+            UserStats::String(s) => write!(f, "{}", s),
+            UserStats::Ratio(a, b) => {
+                if *b == 0 {
+                    write!(f, "{}/{}", a, b)
+                } else {
+                    write!(f, "{}/{} ({}%)", a, b, a * 100 / b)
+                }
+            }
+        }
+    }
+}
+
 /// A simple struct to keep track of client stats
 #[derive(Debug, Clone, Default)]
 pub struct ClientStats {
@@ -30,6 +65,8 @@ pub struct ClientStats {
     pub last_window_time: time::Duration,
     /// The last executions per sec
     pub last_execs_per_sec: f32,
+    /// User-defined stats
+    pub user_stats: HashMap<String, UserStats>,
 
     /// Client performance statistics
     #[cfg(feature = "introspection")]
@@ -90,6 +127,16 @@ impl ClientStats {
         self.last_execs_per_sec as u64
     }
 
+    /// Update the user-defined stat with name and value
+    pub fn update_user_stats(&mut self, name: String, value: UserStats) {
+        self.user_stats.insert(name, value);
+    }
+
+    /// Get a user-defined stat using the name
+    pub fn get_user_stats(&mut self, name: &str) -> Option<&UserStats> {
+        self.user_stats.get(name)
+    }
+
     /// Update the current [`ClientPerfStats`] with the given [`ClientPerfStats`]
     #[cfg(feature = "introspection")]
     pub fn update_introspection_stats(&mut self, introspection_stats: ClientPerfStats) {
@@ -109,7 +156,7 @@ pub trait Stats {
     fn start_time(&mut self) -> time::Duration;
 
     /// show the stats to the user
-    fn display(&mut self, event_msg: String);
+    fn display(&mut self, event_msg: String, sender_id: u32);
 
     /// Amount of elements in the corpus (combined for all children)
     fn corpus_size(&self) -> u64 {
@@ -149,12 +196,55 @@ pub trait Stats {
             self.client_stats_mut().push(ClientStats {
                 last_window_time: current_time(),
                 ..ClientStats::default()
-            })
+            });
         }
         &mut self.client_stats_mut()[client_id as usize]
     }
 }
 
+/// Stats that print exactly nothing.
+/// Not good for debuging, very good for speed.
+pub struct NopStats {
+    start_time: Duration,
+    client_stats: Vec<ClientStats>,
+}
+
+impl Stats for NopStats {
+    /// the client stats, mutable
+    fn client_stats_mut(&mut self) -> &mut Vec<ClientStats> {
+        &mut self.client_stats
+    }
+
+    /// the client stats
+    fn client_stats(&self) -> &[ClientStats] {
+        &self.client_stats
+    }
+
+    /// Time this fuzzing run stated
+    fn start_time(&mut self) -> time::Duration {
+        self.start_time
+    }
+
+    fn display(&mut self, _event_msg: String, _sender_id: u32) {}
+}
+
+impl NopStats {
+    /// Create new [`NopStats`]
+    #[must_use]
+    pub fn new() -> Self {
+        Self {
+            start_time: current_time(),
+            client_stats: vec![],
+        }
+    }
+}
+
+impl Default for NopStats {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 /// Tracking stats during fuzzing.
 #[derive(Clone, Debug)]
 pub struct SimpleStats<F>
@@ -186,10 +276,11 @@ where
         self.start_time
     }
 
-    fn display(&mut self, event_msg: String) {
+    fn display(&mut self, event_msg: String, sender_id: u32) {
         let fmt = format!(
-            "[{}] clients: {}, corpus: {}, objectives: {}, executions: {}, exec/sec: {}",
+            "[{} #{}] clients: {}, corpus: {}, objectives: {}, executions: {}, exec/sec: {}",
             event_msg,
+            sender_id,
             self.client_stats().len(),
             self.corpus_size(),
             self.objective_size(),
@@ -201,14 +292,15 @@ where
         // Only print perf stats if the feature is enabled
         #[cfg(feature = "introspection")]
         {
-            // Print the client performance stats. Skip the Client 0 which is the broker
-            for (i, client) in self.client_stats.iter().skip(1).enumerate() {
-                let fmt = format!("Client {:03}: {}", i + 1, client.introspection_stats);
-                (self.print_fn)(fmt);
-            }
+            // Print the client performance stats.
+            let fmt = format!(
+                "Client {:03}: {}",
+                sender_id, self.client_stats[sender_id as usize].introspection_stats
+            );
+            (self.print_fn)(fmt);
 
             // Separate the spacing just a bit
-            (self.print_fn)("\n".to_string());
+            (self.print_fn)("".to_string());
         }
     }
 }
@@ -265,12 +357,6 @@ macro_rules! mark_feedback_time {
     }};
 }
 
-/// Number of stages in the fuzzer
-pub(crate) const NUM_STAGES: usize = 8;
-
-/// Number of feedback mechanisms to measure for performance
-pub(crate) const NUM_FEEDBACKS: usize = 16;
-
 /// Client performance statistics
 #[derive(Serialize, Deserialize, Debug, Copy, Clone)]
 pub struct ClientPerfStats {
@@ -399,7 +485,7 @@ impl ClientPerfStats {
     /// the current clock counter
     #[must_use]
     pub fn new() -> Self {
-        let start_time = crate::cpu::read_time_counter();
+        let start_time = crate::bolts::cpu::read_time_counter();
 
         Self {
             start_time,
@@ -424,7 +510,7 @@ impl ClientPerfStats {
     /// Start a timer with the current time counter
     #[inline]
     pub fn start_timer(&mut self) {
-        self.timer_start = Some(crate::cpu::read_time_counter());
+        self.timer_start = Some(crate::bolts::cpu::read_time_counter());
     }
 
     /// Update the current [`ClientPerfStats`] with the given [`ClientPerfStats`]
@@ -451,7 +537,7 @@ impl ClientPerfStats {
             }
             Some(timer_start) => {
                 // Calculate the elapsed time
-                let elapsed = crate::cpu::read_time_counter() - timer_start;
+                let elapsed = crate::bolts::cpu::read_time_counter() - timer_start;
 
                 // Reset the timer
                 self.timer_start = None;
@@ -469,7 +555,7 @@ impl ClientPerfStats {
         let elapsed = self.mark_time();
 
         // Add the time to the scheduler stat
-        self.update_scheduler(elapsed)
+        self.update_scheduler(elapsed);
     }
 
     /// Update the time spent in the scheduler with the elapsed time that we have seen
@@ -685,6 +771,7 @@ impl core::fmt::Display for ClientPerfStats {
 
 #[cfg(feature = "introspection")]
 impl Default for ClientPerfStats {
+    #[must_use]
     fn default() -> Self {
         Self::new()
     }
